/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.DisplayBean.NodeShape;

import java.awt.Shape;
import java.awt.geom.AffineTransform;
import java.awt.geom.Point2D;
import java.awt.geom.Rectangle2D;
import java.awt.geom.RoundRectangle2D;

import ca.uvic.csr.shrimp.DisplayBean.DisplayConstants;
import ca.uvic.csr.shrimp.util.GeometryUtils;

/**
 * @author Rob Lintern
 */
public class RoundedRectangleNodeShape extends AbstractNodeShape {
	
	public static final String ROUNDED_RECTANGLE_NODE_SHAPE = "Rounded Rectangle";
	private static final double MIN_DIM_TO_RADIUS_RATIO = 3.0d;
	
	public RoundedRectangleNodeShape(String name) {
		super(name);
	}
	
	public RoundedRectangleNodeShape() {
		this(ROUNDED_RECTANGLE_NODE_SHAPE);
	}
	
	/** 
	 * @see ca.uvic.csr.shrimp.DisplayBean.NodeShape.AbstractNodeShape#clone()
	 */
	public Object clone() {
		NodeShape shape = new RoundedRectangleNodeShape();
		return shape;
	}
	
	public Rectangle2D.Double getInnerBounds(Rectangle2D outerBounds) {
		RoundRectangle2D shape = (RoundRectangle2D) getShape(outerBounds);
		double arcH = shape.getArcHeight()/4.0d;
		double insetW = 1; //arcW/Math.sqrt(2.0d);
		double insetH = arcH * 2; //arcH/Math.sqrt(2.0d);
		double x = shape.getBounds2D().getX() + insetW;
		double y = shape.getBounds2D().getY() + insetH;
		double w = shape.getBounds2D().getWidth() - 2.0d*insetW;
		double h = shape.getBounds2D().getHeight() - 2.0d*insetH;
		return new Rectangle2D.Double(x,y,w,h);
	}

	public double getTerminalAttachPoint(double terminalPositionAngle, Rectangle2D.Double globalBounds, AffineTransform localToGlobalTx, Point2D.Double arcAttachPoint) {
		double terminalTheta = 0.0;
		RoundRectangle2D shape = (RoundRectangle2D) getShape(globalBounds);
		double nw = globalBounds.getWidth(); // node width
		double nh = globalBounds.getHeight(); // node height
		double nhalfw = nw/2.0d; // half node width
		double nhalfh = nh/2.0d; // half node height
		double tanTheta = Math.tan(terminalPositionAngle);
		
		double r = shape.getArcWidth()/2.0d; // * localToGlobalTx.getScaleX(); // corner radius, in global coordinates
		double segA = nhalfh - r;
		double segB = nhalfw - r;

		/* 
		 * Idea: A rounded rectangle node has four line and four arc seqments. 
		 * Firstly, see which segments of the side intersects a line with 
		 * an angle of "theta", originating at the centre of the node. 
		 * Secondly, see where exactly on the side this intersection happens.
		 * Note: The y axis is inverted like screen coordinates, and theta is between -PI and +PI
		 * 
		 * 						 -Y	
		 *                          
		 *                         | 
		 *                         | -PI/2
		 *                         | 
		 *                         |
		 *                   |     |     |  |
		 *                   |segB |segB |r |
		 *                   |_____|_____|__|_________  
		 *                   /     |     \  |  r
		 * 	               /       |       \|_________
		 *                |        |        |
		 *                |        |        |
		 *                |        |        |  segA
		 *                |        |        |
		 *       -PI      |        |        |              - 0
		 * -X  ___________|________|________|__________________   +X
		 *                |        |        |
		 *       +PI      |        |        |              + 0
		 *                |        |        |  segA
		 *                |        |        |
		 *                |        |        |
		 *                |        |        |_________
		 *                 \       |       /
		 *                   \_____|_____/   
		 *                         | 
		 *                         |
		 *                         |
		 *                         | +PI/2
		 * 
		 *                       +Y
		 */
		double alpha = Math.atan2(segA, nhalfw); // angle in radians between line from centre of node to lower right end of segA and x axis
		double beta = Math.atan2(segB, nhalfh); // angle in radians between line from centre of node to lower right end of segB and y axis
		double gamma = DisplayConstants.HALF_PI - alpha - beta; // remainder, ie. the angle between lines from center of node to endpoints of the rounded corner arc
		double x = 0.0d;
		double y = 0.0d;
		if (Math.abs(terminalPositionAngle) <= alpha) { 
			// right side
			x = nhalfw;
			y = x * tanTheta;
			terminalTheta = 0.0;
		} else if (Math.abs(terminalPositionAngle) <= (alpha + gamma)) {
			// upper or lower right corner
			Point2D [] result = new Point2D [2];
			if (terminalPositionAngle < 0) {				
				// upper right corner
				GeometryUtils.circleLineIntersection(terminalPositionAngle, r, segB, -segA, result);
				x = result[0].getX();
				y = result[0].getY();
				terminalTheta = terminalPositionAngle;				
			} else {
				// lower right corner
				GeometryUtils.circleLineIntersection(terminalPositionAngle, r, segB, segA, result);
				x = result[0].getX();
				y = result[0].getY();
				terminalTheta = terminalPositionAngle;				
			}
		} else if (Math.abs(terminalPositionAngle) <= (alpha + gamma + 2.0d*beta)) {
			// top or bottom
			y = (terminalPositionAngle < 0) ? -nhalfh : nhalfh;
			x = y / tanTheta;
			terminalTheta = (terminalPositionAngle < 0) ? DisplayConstants.NEG_HALF_PI : DisplayConstants.HALF_PI;
		} else if (Math.abs(terminalPositionAngle) <= (alpha + gamma + 2.0d*beta + gamma)) {
			// upper or lower left corner
			Point2D [] result = new Point2D [2];
			if (terminalPositionAngle < 0) {				
				// upper left corner
				GeometryUtils.circleLineIntersection(terminalPositionAngle, r, -segB, -segA, result);
				x = result[1].getX();
				y = result[1].getY();
				terminalTheta = terminalPositionAngle;				
			} else {
				// lower left corner
				GeometryUtils.circleLineIntersection(terminalPositionAngle, r, -segB, segA, result);
				x = result[1].getX();
				y = result[1].getY();
				terminalTheta = terminalPositionAngle;				
			}
		} else {
			// left side
			x = -nhalfw;
			y = x * tanTheta;
			terminalTheta = DisplayConstants.PI;
		}
		arcAttachPoint.setLocation(x,y);
		return terminalTheta;
	}
	
	/** 
	 * @see ca.uvic.csr.shrimp.DisplayBean.NodeShape.NodeShape#getShape(java.awt.geom.Rectangle2D)
	 */
	public Shape getShape(Rectangle2D bounds) {
		double radius = Math.min(bounds.getWidth(), bounds.getHeight())/MIN_DIM_TO_RADIUS_RATIO;
		RoundRectangle2D shape = new RoundRectangle2D.Double (bounds.getX(), bounds.getY(), bounds.getWidth(), bounds.getHeight(), radius, radius);
		return shape;
	}


	

}
