/**
 * Copyright 1998-2007, CHISEL Group, University of Victoria, Victoria, BC, Canada.
 * All rights reserved.
 */
package ca.uvic.csr.shrimp.gui.ShrimpView;

import java.awt.Container;
import java.io.File;

import ca.uvic.csr.shrimp.JavaDomainConstants;
import ca.uvic.csr.shrimp.ShrimpConstants;
import ca.uvic.csr.shrimp.ShrimpApplication.ApplicationAccessor;
import ca.uvic.csr.shrimp.ShrimpApplication.ShrimpApplication;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectAdapter;
import ca.uvic.csr.shrimp.ShrimpProject.ShrimpProjectEvent;
import ca.uvic.csr.shrimp.gui.LoadPreviousViewDialog;
import ca.uvic.csr.shrimp.gui.FilmStrip.ShrimpSnapShot;
import ca.uvic.csr.shrimp.jambalaya.JambalayaApplication;
import ca.uvic.csr.shrimp.jambalaya.ProtegeDataBean.OBODataBean;
import ca.uvic.csr.shrimp.usercontrols.DefaultViewAction;
import ca.uvic.csr.shrimp.usercontrols.NestedTreemapViewAction;
import ca.uvic.csr.shrimp.usercontrols.UserAction;

/**
 * This abstract class provides some common code for create and disposing of a Shrimp View.
 * 
 * @author Rob Lintern, Chris Callendar
 */
public abstract class AbstractShrimpViewFactory implements ShrimpViewFactory {
	
	protected Container shrimpViewContainer;
	protected ShrimpView shrimpView;
	protected ShrimpProject project;

	/** Default quick view names displayed by stand-alone SHriMP */
    protected final String[] JAVA_QUICK_VIEW_ACTION_NAMES = {
            DefaultViewAction.ACTION_NAME, 
            JavaDomainConstants.JAVA_QUICK_VIEW_CLASS_INTERFACE_HIERARCHY,
            JavaDomainConstants.JAVA_QUICK_VIEW_CALL_GRAPH,
            JavaDomainConstants.JAVA_QUICK_VIEW_CONTROL_FLOW_GRAPH,
            JavaDomainConstants.JAVA_QUICK_VIEW_SEQUENCE_DIAGRAM,
            JavaDomainConstants.JAVA_QUICK_VIEW_PACKAGE_DEPENDENCIES_CALLS_ACCESSES,
            NestedTreemapViewAction.ACTION_NAME,
    };
    
    /** Default quick view names for Protege projects. */
	protected final String[] PROTEGE_QUICK_VIEW_ACTION_NAMES = {
            DefaultViewAction.ACTION_NAME, 
            JambalayaApplication.QUICK_VIEW_CLASS_TREE, 
            /*JambalayaApplication.QUICK_VIEW_CLASS_INDIVIDUAL_TREE,*/ 
            JambalayaApplication.QUICK_VIEW_PART_OF_TREE, 
            JambalayaApplication.QUICK_VIEW_CLASS_INSTANCE_TREE, 
            JambalayaApplication.QUICK_VIEW_DOMAIN_RANGE,
            //JambalayaApplication.QUICK_VIEW_QUERY_VIEW,	// depends on Jambalaya/Protege
            NestedTreemapViewAction.ACTION_NAME,
    };
	
	/** Default quick view names for OBO projects. */
	protected final String[] OBO_QUICK_VIEW_ACTION_NAMES = {
		DefaultViewAction.ACTION_NAME,
		OBODataBean.IS_A_HIERARCHY,
		OBODataBean.PART_OF_HIERARCHY,
        NestedTreemapViewAction.ACTION_NAME,
	};
	
	/** 
	 * @see ca.uvic.csr.shrimp.gui.ShrimpView.ShrimpViewFactory#createShrimpView(ca.uvic.csr.shrimp.ShrimpProject.ShrimpProject, java.awt.Container)
	 */
	public ShrimpView createShrimpView(final ShrimpProject project, Container shrimpViewContainer) {
		this.project = project;
		this.shrimpViewContainer = shrimpViewContainer;
		final ShrimpApplication application = ApplicationAccessor.getApplication();
		//create a shrimpView object, initialize the beans, then call init
		shrimpView = new ShrimpView(project);			
		shrimpView.createShrimpViewBeansForProject();
		shrimpView.init();
		shrimpView.initShrimpViewBeansForProject();
		
		project.addTool(ShrimpProject.SHRIMP_VIEW, shrimpView);
		projectSpecificCreateShrimpView();
		project.addProjectListener(new ShrimpProjectAdapter() {
			public void projectClosing(ShrimpProjectEvent event) {
				// must be done before the shrimp view is disposed
				takeClosingSnapShot(event.getProject());
				disposeShrimpView();
				project.removeProjectListener(this);
			}
		});
		
		final String fileName =  ShrimpSnapShot.getSnapShotName(project); 
		File snapShotFile = new File(fileName);
		try {
            if (snapShotFile.exists()) {
            	LoadPreviousViewDialog loadPreviousViewDialog = new LoadPreviousViewDialog(application, project, snapShotFile); 
            	loadPreviousViewDialog.showDialog();
            	if (loadPreviousViewDialog.getLoadPreviousView()) {
            		startAtSnapShotState(loadPreviousViewDialog.getSnapShot());
            	} else {
            		startAtDefaultState();
            	}
            } else {
            	startAtDefaultState();
            }
        } catch (Throwable e) {
            //e.printStackTrace();
        	startAtDefaultState();
        }
		return shrimpView;
	}
	
	/**
	 * Takes a closing snapshot.
	 */
	protected void takeClosingSnapShot(ShrimpProject project) {
		ShrimpSnapShot.takeSnapShot(project, shrimpView);
	}

	/**
	 * Defines the default actions to take on first loading the ShrimpView. 
	 * Can be overriden if other actions are useful in different projects.
	 */
	protected void startAtDefaultState () {
	    UserAction defaultViewAction = (UserAction) project.getActionManager().getAction(DefaultViewAction.ACTION_NAME, ShrimpConstants.MENU_QUICK_VIEWS);
	    if (defaultViewAction != null) {
	        defaultViewAction.startAction();
	    } else {
	        (new DefaultViewAction(project)).startAction();
	    }
	}
	
	private void startAtSnapShotState (final ShrimpSnapShot snapShot)  {
		snapShot.revertViewToSnapShotState(project, shrimpView);
	}

	/**
	 * Disposes of the Shrimp View and removes it from its container.
	 */
	protected void disposeShrimpView() {
		projectSpecificDisposeShrimpView();
		//shrimpView.disposeTool();	// already done when project closing event is fired
		shrimpViewContainer.removeAll();
		shrimpViewContainer = null;
	}

	/**
	 * Implement this method to provide any domain or project specific initialization of the Shrimp View. 
	 */
	protected abstract void projectSpecificCreateShrimpView ();
	
	/**
	 * Implement this method to provide any domain or project specific "finalization" of the Shrimp View. 
	 */
	protected abstract void projectSpecificDisposeShrimpView ();
	
	
}